from __future__ import absolute_import, unicode_literals

from deux.abstract_models import AbstractMultiFactorAuth


class MultiFactorAuth(AbstractMultiFactorAuth):
    """
    class::MultiFactorAuth()

    Blank extension of ``AbstractMultiFactorAuth`` that is used as the
    default model in this package.
    """
    pass
